@@ -3,7 +3,7 @@ package App::Maisha::Plugin::Twitter;
 use strict;
 use warnings;
 
-our $VERSION = '0.17';
+our $VERSION = '0.18';
 
 #----------------------------------------------------------------------------
 # Library Modules
@@ -17,7 +17,7 @@ use Storable;
 #----------------------------------------------------------------------------
 # Accessors
 
-__PACKAGE__->mk_accessors($_) for qw(api users);
+__PACKAGE__->mk_accessors($_) for qw(api users config);
 
 #----------------------------------------------------------------------------
 # Public API
@@ -52,8 +52,7 @@ sub login {
 
     eval {
         $api = Net::Twitter->new(
-#            traits              => [qw/API::REST OAuth WrapError/],
-            traits              => [qw/API::REST OAuth/],
+            traits              => [qw/API::Search API::REST OAuth/],
             consumer_key        => $self->{consumer_key},
             consumer_secret     => $self->{consumer_secret},
             ssl                 => 1
@@ -81,14 +80,26 @@ sub login {
                 my $pin = <STDIN>; # wait for input
                 chomp $pin;
 
+                unless($pin) {
+                    warn "No PIN provided, Maisha will not be able to access Twitter account until authorized to do so\n";
+                    return 0;
+                }
+
                 # request_access_token stores the tokens in $nt AND returns them
-                my @access_tokens = $api->request_access_token(verifier => $pin);
+                my $access_tokens = {};
+                my @access_tokens;
+                eval { @access_tokens = $api->request_access_token(verifier => $pin) };
+                unless(@access_tokens) {
+                    warn "Invalid PIN provided, Maisha will not be able to access your Twitter account until authorized to do so\n";
+                    return 0;
+                }
                 $access_tokens->{$config->{username}} = \@access_tokens;
 
                 mkpath( $config->{home} . '/.maisha' );
 
                 # save the access tokens
                 store $access_tokens, $datafile;
+                chmod 0640, $datafile;  # make sure it has reasonable permissions
             }
         };
 
@@ -99,6 +110,7 @@ sub login {
     }
 
     $self->api($api);
+    $self->config($config);
 
     if(!$config->{test}) {
         print "...building user cache for Twitter\n";
@@ -122,9 +134,62 @@ sub _build_users {
     $self->users(\%users);
 }
 
-sub api_update {
+sub api_reauthorize {
+    my $self    = shift;
+    my $config  = $self->config;
+    my $api     = $self->api;
+
+    # for testing purposes we don't want to login
+    if(!$config->{test}) {
+        eval {
+            my $datafile = $config->{home} . '/.maisha/twitter.dat';
+
+            my $auth_url = $api->get_authorization_url;
+            print "Please re-authorize this application at: $auth_url\nThen, enter the PIN# provided to continue: ";
+
+            my $pin = <STDIN>; # wait for input
+            chomp $pin;
+
+            unless($pin) {
+                warn "No PIN provided, Maisha will not be able to access direct messages until reauthorization is completed\n";
+                return 0;
+            }
+
+            # request_access_token stores the tokens in $nt AND returns them
+            my $access_tokens = {};
+            my @access_tokens;
+            eval { @access_tokens = $api->request_access_token(verifier => $pin) };
+            unless(@access_tokens) {
+                warn "Invalid PIN provided, Maisha will not be able to access direct messages until reauthorization is completed\n";
+                return 0;
+            }
+            $access_tokens->{$config->{username}} = \@access_tokens;
+
+            unlink $datafile;
+            mkpath( $config->{home} . '/.maisha' );
+
+            # save the access tokens
+            store $access_tokens, $datafile;
+            chmod 0640, $datafile;  # make sure it has reasonable permissions
+        };
+
+        if($@) {
+            warn "Unable to login to Twitter\n";
+            return 0;
+        }
+    }
+
+    return 1;
+}
+
+sub api_follow {
     my $self = shift;
-    $self->api->update(@_);
+    $self->api->create_friend(@_);
+}
+
+sub api_unfollow {
+    my $self = shift;
+    $self->api->destroy_friend(@_);
 }
 
 sub api_user {
@@ -139,30 +204,35 @@ sub api_user_timeline {
 
 sub api_friends {
     my $self = shift;
-    $self->api->following();
+    $self->api->following(@_);
 }
 
 sub api_friends_timeline {
     my $self = shift;
-    $self->api->following_timeline();
+    $self->api->following_timeline(@_);
 }
 
 sub api_public_timeline {
     my $self = shift;
-    $self->api->public_timeline();
+    $self->api->public_timeline(@_);
 }
 
 sub api_followers {
     my $self = shift;
-    $self->api->followers();
+    $self->api->followers(@_);
 
     # below is meant to be the same as the above, but it isn't :(
     #$self->api->lookup_users( { user_id => $self->api->followers_ids() } );
 }
 
+sub api_update {
+    my $self = shift;
+    $self->api->update(@_);
+}
+
 sub api_replies {
     my $self = shift;
-    $self->api->replies();
+    $self->api->replies(@_);
 }
 
 sub api_send_message {
@@ -172,22 +242,17 @@ sub api_send_message {
 
 sub api_direct_messages_to {
     my $self = shift;
-    $self->api->direct_messages();
+    $self->api->direct_messages(@_);
 }
 
 sub api_direct_messages_from {
     my $self = shift;
-    $self->api->sent_direct_messages();
+    $self->api->sent_direct_messages(@_);
 }
 
-sub api_follow {
+sub api_search {
     my $self = shift;
-    $self->api->create_friend(@_);
-}
-
-sub api_unfollow {
-    my $self = shift;
-    $self->api->destroy_friend(@_);
+    $self->api->search(@_);
 }
 
 1;
@@ -235,6 +300,8 @@ The API methods are used to interface to with the Twitter API.
 
 =over 4
 
+=item * api_reauthorize
+
 =item * api_follow
 
 =item * api_unfollow
@@ -261,6 +328,8 @@ The API methods are used to interface to with the Twitter API.
 
 =item * api_direct_messages_from
 
+=item * api_search
+
 =back
 
 =head1 AUTHENTICATION
